from expetator.monitors.kwollect import Power as Kwollect
from expetator.monitors.lperf import Lperf
from expetator.monitors.mojitos import Mojitos
from expetator.monitors.powergpu import Power as PowerGpu
