from expetator.benchmarks.cpubench import CpuBench
from expetator.benchmarks.genericbench import GenericBench
from expetator.benchmarks.gpucpubench import GpuCpuBench
from expetator.benchmarks.gpumembench import GpuMemBench
from expetator.benchmarks.gromacsbench import GromacsBench
from expetator.benchmarks.membench import MemBench
from expetator.benchmarks.mpibench import MpiBench
from expetator.benchmarks.npbbench import NpbBench
from expetator.benchmarks.percentagebench import PercentageBench
from expetator.benchmarks.sleepbench import SleepBench
from expetator.benchmarks.watermark import WaterMark
