from expetator.leverages.dvfs import Dvfs
from expetator.leverages.gpuclock import GpuClock
from expetator.leverages.gpupow import GpuPower
from expetator.leverages.neosched import NeoSched
from expetator.leverages.nodeepsleep import Nodeepsleep
from expetator.leverages.pct import Pct
from expetator.leverages.powercap import Powercap
from expetator.leverages.template import Template
